/* Flux - 6502 CPU Emulator
   version 0.1.1
   by DarkOrk - DarkOrk@digicron.com - UIN: 1695704

   Made with DJGPP, RHIDE, and Notepad

   Released under GNU GPL -=- http://www.gnu.org/
*/

#include <stdio.h>
#include <conio.h>
#include <bios.h>

/*#include "JGmouse.h"*/
#include "flux.h"

#define FALSE 0
#define TRUE  1

void cpu(FILE* f);

struct fluxPC {
  unsigned char opcode;      /* the current opcode */
  char databytea, databyteb; /* data taken from the 6502 binary used for the current opcode being emulated */
  char a;                    /* the accumulator */
  char x;                    /* x index register */
  char y;                    /* y index register */
  char s;                    /* stack pointer */
  char p;                    /* processor status flags */
  char sbradrslst[128];      /* subroutine address list */
  int  saldex;               /* subroutine address list index (keep track of where we are in the array) */
  char PC;                   /* program counter */
  char memory[4096];         /* memory array */
} PC;

int done = FALSE;

int main (int argc, char* argv[])
{
  FILE* binfile;               /* the binary to use for emulation */
  char kbinp;                  /* buffer for input from the keyboard */
  char memaddress;             /* the memory address to dump (specified by user) */
  unsigned int dumpmemdex = 0; /* index variable used for dumping (entire) memory */

  /* JGmouse functions - get the mouse ready to use */
  /*JG_init_mouse();
  JG_show_mouse();   !!Not needed yet - once I make a GUI...*/

  /* clear the screen - get rid of the prompt */
  clrscr();

  /* Bit 5 of P is always 1 */
  PC.p = bit5; /* bit5 = 32 defined in flux.h */

  /* Dev Stuff */
  /* End Dev Stuff */

  /* open the binary to use for emulation */
  if(!(binfile = fopen(argv[1],"rb"))) {
    printf("Error opening %s!", argv[1]);
    return 1;
  }

  /* make sure a few variables in the fluxPC struct equal 0 */
  PC.PC = 0;
  PC.saldex = 0;

  /* Flux's main program loop */
  while(!done) {
    /* display the flux user interface */
    printf("\nA: $%x", PC.a);
    printf("\nX: $%x", PC.x);
    printf("\nY: $%x", PC.y);
    printf("\nS: $%x", PC.s);
    printf("\nP ($%x) -", PC.p);
    printf(" C: %d", (PC.p & C));
    printf("  Z: %d", (PC.p & Z));
    printf("  I: %d", (PC.p & I));
    printf("  D: %d", (PC.p & D));
    printf("  B: %d", (PC.p & B));
    printf("  bit5: %d", (PC.p & bit5));
    printf("  V: %d", (PC.p & V));
    printf("  N: %d", (PC.p & N));
    printf("\nCurrent PC: $%x", PC.PC);
    printf("\nCurrent Opcode: $%x - %s (%s)", PC.opcode, opcodenames[PC.opcode], addressing[PC.opcode]);
    printf("\nJSR List: $%x", PC.sbradrslst[PC.saldex-1]);

    printf("\n\n");
    printf("S   - Step to next instruction\n");
    printf("M   - Look at memory address\n");
    printf("D   - Dump memory addresses that are being used\n");
    printf("Esc - Exit");

    fflush(stdout);

    /* get keyboard input, clear the screen, do what the user wanted */
    kbinp = getch(); 
    clrscr();

    if(kbinp == 83 || kbinp == 115) {
      PC.opcode = fgetc(binfile);
      PC.PC++;
      cpu(binfile);
    }

    if(kbinp == 109 || kbinp == 77) {
      printf("Enter hex address of memory to dump: ");
      scanf("%x",&memaddress);
      printf("%x = %x",memaddress,PC.memory[memaddress]);

    }

    if(kbinp == 68 || kbinp == 100) {
      printf("Dumping memory that is being used...\n");
      for(dumpmemdex = 0; dumpmemdex < 4096; dumpmemdex++) {
        if(PC.memory[dumpmemdex])
          printf("$%x = %x\n", dumpmemdex, PC.memory[dumpmemdex]);
      }
      printf("Press a key...");
      getch();
      clrscr();
    }

    if(kbinp == 27)
      done = TRUE;

    kbinp = 0;
  }

  return 0;
}

void cpu(FILE* f)
{
  int ttime=0;
  int jbytes=0;
  switch(PC.opcode) {
    case ASL:
      PC.a = (PC.a<<1);
    break;
    case CLD:
      PC.p &= Dmsk_clr;
    break;
    case DEC:
      PC.a--;
    break;
    case DEX:
      PC.x--;
    break;
    case DEY:
      PC.y--;
    break;
    case INC:
      PC.a++;
    break;
    case INX:
      PC.x++;
    break;
    case INY:
      PC.y++;
    break;
    case JMP:
      PC.databytea = fgetc(f); PC.PC++;
      PC.databyteb = fgetc(f); PC.PC++;
      jbytes = PC.databytea; // add 2nd byte
      PC.PC = jbytes;
      fseek(f, jbytes, SEEK_SET);
    break;
    case JSR:
      PC.databytea = fgetc(f); PC.PC++;
      PC.databyteb = fgetc(f); PC.PC++;
      PC.sbradrslst[PC.saldex] = PC.PC; PC.saldex++;
      jbytes = PC.databytea; // add 2nd byte
      PC.PC = jbytes;
      fseek(f, jbytes, SEEK_SET);
    break;
    case LDA:
      PC.databytea = fgetc(f); PC.PC++;
      PC.a = PC.databytea;
    break;
    case LDAx:
      PC.databytea = fgetc(f); PC.PC++;
      PC.a = PC.memory[(PC.databytea + PC.x)];
    break;
    case LDAma:
      PC.databytea = fgetc(f); PC.PC++;
      PC.a = PC.memory[PC.databytea];
    break;
    case LDAmb:
      PC.databytea = fgetc(f); PC.PC++;
      PC.databyteb = fgetc(f); PC.PC++;
      /* Add functionality */
    break;
    case LDX:
      PC.databytea = fgetc(f); PC.PC++;
      PC.x = PC.databytea;
    break;
    case LDY:
      PC.databytea = fgetc(f); PC.PC++;
      PC.y = PC.databytea;
    break;
    case LSR:
      PC.a = (PC.a>>1);
    break;
    case NOP:
      for(ttime; ttime>256; ttime++)
        PC.databytea = ttime;
    break;
    case RTS:
      PC.saldex--;
      PC.PC = PC.sbradrslst[PC.saldex];
      fseek(f, PC.PC, SEEK_SET);
    break;
    case STAa:
      PC.databytea = fgetc(f); PC.PC++;
      PC.memory[PC.databytea] = PC.a;
    break;
    case STAb:
      PC.databytea = fgetc(f); PC.PC++;
      PC.databyteb = fgetc(f); PC.PC++;
      PC.memory[PC.databytea] = PC.a; /**FIX**/
    break;
    case STXa:
      PC.databytea = fgetc(f); PC.PC++;
      PC.memory[PC.databytea] = PC.x;
    break;
    case STXb:
      PC.databytea = fgetc(f); PC.PC++;
      PC.databyteb = fgetc(f); PC.PC++;
      PC.memory[PC.databytea] = PC.x; /**FIX**/
    break;
    case STYa:
      PC.databytea = fgetc(f); PC.PC++;
      PC.memory[PC.databytea] = PC.y;
    break;
    case STYb:
      PC.databytea = fgetc(f); PC.PC++;
      PC.databyteb = fgetc(f); PC.PC++;
      PC.memory[PC.databytea] = PC.y; /**FIX**/
    break;
    case TAX:
      PC.x = PC.a;
    break;
    case TAY:
      PC.y = PC.a;
    break;
    case TXA:
      PC.a = PC.x;
    break;
    case TYA:
      PC.a = PC.y;
    break;
  }
}
