/* Processor status bits/masks */
#define C     1
#define Z     2
#define I     4
#define D     8
#define B    16
#define bit5 32
#define V    64
#define N   128

#define Cmsk_set 0x01 /* 00000001 */
#define Zmsk_set 0x02 /* 00000010 */
#define Imsk_set 0x04 /* 00000100 */
#define Dmsk_set 0x08 /* 00001000 */
#define Bmsk_set 0x10 /* 00010000 */
/* Flux - 6502 CPU Emulator
   version 0.1.1
   by DarkOrk - DarkOrk@digicron.com - UIN: 1695704

   Made with DJGPP, RHIDE, and Notepad

   Released under GNU GPL -=- http://www.gnu.org/
*/

/* bit5 !used    0x20    00100000 */
#define Vmsk_set 0x40 /* 01000000 */
#define Nmsk_set 0x80 /* 10000000 */

#define Cmsk_clr 0xFE /* 11111110 */
#define Zmsk_clr 0xFD /* 11111101 */
#define Imsk_clr 0xFB /* 11111011 */
#define Dmsk_clr 0xF7 /* 11110111 */
#define Bmsk_clr 0xEF /* 11101111 */
/* bit5 !used    0xDF    11011111 */
#define Vmsk_clr 0xBF /* 10111111 */
#define Nmsk_clr 0x7F /* 01111111 */


// Mnemonics
#define ASL 0x0A

#define CLD 0xD8

#define DEC 0x3A

#define DEX 0xCA
#define DEY 0x88

#define INC 0x1A

#define INX 0xE8
#define INY 0xC8

#define JMP 0x4C

#define JSR 0x20

#define LDA   0xA9
#define LDAx  0xA1
#define LDAma 0xA5
#define LDAmb 0xAD

#define LDX 0xA2

#define LDY 0xA0

#define LSR 0x4A

#define NOP 0xEA

#define RTS 0x60

#define SEI 0x78

#define STAa 0x85
#define STAb 0x8D

#define STXa 0x86
#define STXb 0x8E

#define STYa 0x84
#define STYb 0x8C

#define TAX 0xAA
#define TAY 0xA8
#define TXA 0x8A
#define TYA 0x98

char opcodenames[256][4] = {
/*0x00*/ "",
/*0x01*/ "",
/*0x02*/ "",
/*0x03*/ "",
/*0x04*/ "",
/*0x05*/ "",
/*0x06*/ "",
/*0x07*/ "",
/*0x08*/ "",
/*0x09*/ "",
/*0x0A*/ "ASL",
/*0x0B*/ "",
/*0x0C*/ "",
/*0x0D*/ "",
/*0x0E*/ "",
/*0x0F*/ "",
/*0x10*/ "",
/*0x11*/ "",
/*0x12*/ "",
/*0x13*/ "",
/*0x14*/ "",
/*0x15*/ "",
/*0x16*/ "",
/*0x17*/ "",
/*0x18*/ "",
/*0x19*/ "",
/*0x1A*/ "",
/*0x1B*/ "",
/*0x1C*/ "",
/*0x1D*/ "",
/*0x1E*/ "",
/*0x1F*/ "",
/*0x20*/ "JSR",
/*0x21*/ "",
/*0x22*/ "",
/*0x23*/ "",
/*0x24*/ "",
/*0x25*/ "",
/*0x26*/ "",
/*0x27*/ "",
/*0x28*/ "",
/*0x29*/ "",
/*0x2A*/ "",
/*0x2B*/ "",
/*0x2C*/ "",
/*0x2D*/ "",
/*0x2E*/ "",
/*0x2F*/ "",
/*0x30*/ "",
/*0x31*/ "",
/*0x32*/ "",
/*0x33*/ "",
/*0x34*/ "",
/*0x35*/ "",
/*0x36*/ "",
/*0x37*/ "",
/*0x38*/ "",
/*0x39*/ "",
/*0x3A*/ "DEC",
/*0x3B*/ "",
/*0x3C*/ "",
/*0x3D*/ "",
/*0x3E*/ "",
/*0x3F*/ "",
/*0x40*/ "",
/*0x41*/ "",
/*0x42*/ "",
/*0x43*/ "",
/*0x44*/ "",
/*0x45*/ "",
/*0x46*/ "",
/*0x47*/ "",
/*0x48*/ "",
/*0x49*/ "",
/*0x4A*/ "",
/*0x4B*/ "",
/*0x4C*/ "JMP",
/*0x4D*/ "",
/*0x4E*/ "",
/*0x4F*/ "",
/*0x50*/ "",
/*0x51*/ "",
/*0x52*/ "",
/*0x53*/ "",
/*0x54*/ "",
/*0x55*/ "",
/*0x56*/ "",
/*0x57*/ "",
/*0x58*/ "",
/*0x59*/ "",
/*0x5A*/ "",
/*0x5B*/ "",
/*0x5C*/ "",
/*0x5D*/ "",
/*0x5E*/ "",
/*0x5F*/ "",
/*0x60*/ "RTS",
/*0x61*/ "",
/*0x62*/ "",
/*0x63*/ "",
/*0x64*/ "",
/*0x65*/ "",
/*0x66*/ "",
/*0x67*/ "",
/*0x68*/ "",
/*0x69*/ "",
/*0x6A*/ "",
/*0x6B*/ "",
/*0x6C*/ "",
/*0x6D*/ "",
/*0x6E*/ "",
/*0x6F*/ "",
/*0x70*/ "",
/*0x71*/ "",
/*0x72*/ "",
/*0x73*/ "",
/*0x74*/ "",
/*0x75*/ "",
/*0x76*/ "",
/*0x77*/ "",
/*0x78*/ "",
/*0x79*/ "",
/*0x7A*/ "",
/*0x7B*/ "",
/*0x7C*/ "",
/*0x7D*/ "",
/*0x7E*/ "",
/*0x7F*/ "",
/*0x80*/ "",
/*0x81*/ "",
/*0x82*/ "",
/*0x83*/ "",
/*0x84*/ "STY",
/*0x85*/ "",
/*0x86*/ "",
/*0x87*/ "",
/*0x88*/ "DEY",
/*0x89*/ "",
/*0x8A*/ "",
/*0x8B*/ "",
/*0x8C*/ "",
/*0x8D*/ "",
/*0x8E*/ "",
/*0x8F*/ "",
/*0x90*/ "",
/*0x91*/ "",
/*0x92*/ "",
/*0x93*/ "",
/*0x94*/ "",
/*0x95*/ "",
/*0x96*/ "",
/*0x97*/ "",
/*0x98*/ "",
/*0x99*/ "",
/*0x9A*/ "",
/*0x9B*/ "",
/*0x9C*/ "",
/*0x9D*/ "",
/*0x9E*/ "",
/*0x9F*/ "",
/*0xA0*/ "",
/*0xA1*/ "",
/*0xA2*/ "",
/*0xA3*/ "",
/*0xA4*/ "",
/*0xA5*/ "",
/*0xA6*/ "",
/*0xA7*/ "",
/*0xA8*/ "",
/*0xA9*/ "LDA",
/*0xAA*/ "",
/*0xAB*/ "",
/*0xAC*/ "",
/*0xAD*/ "",
/*0xAE*/ "",
/*0xAF*/ "",
/*0xB0*/ "",
/*0xB1*/ "",
/*0xB2*/ "",
/*0xB3*/ "",
/*0xB4*/ "",
/*0xB5*/ "",
/*0xB6*/ "",
/*0xB7*/ "",
/*0xB8*/ "",
/*0xB9*/ "",
/*0xBA*/ "",
/*0xBB*/ "",
/*0xBC*/ "",
/*0xBD*/ "",
/*0xBE*/ "",
/*0xBF*/ "",
/*0xC0*/ "",
/*0xC1*/ "",
/*0xC2*/ "",
/*0xC3*/ "",
/*0xC4*/ "",
/*0xC5*/ "",
/*0xC6*/ "",
/*0xC7*/ "",
/*0xC8*/ "",
/*0xC9*/ "",
/*0xCA*/ "DEX",
/*0xCB*/ "",
/*0xCC*/ "",
/*0xCD*/ "",
/*0xCE*/ "",
/*0xCF*/ "",
/*0xD0*/ "",
/*0xD1*/ "",
/*0xD2*/ "",
/*0xD3*/ "",
/*0xD4*/ "",
/*0xD5*/ "",
/*0xD6*/ "",
/*0xD7*/ "",
/*0xD8*/ "CLD",
/*0xD9*/ "",
/*0xDA*/ "",
/*0xDB*/ "",
/*0xDC*/ "",
/*0xDD*/ "",
/*0xDE*/ "",
/*0xDF*/ "",
/*0xE0*/ "",
/*0xE1*/ "",
/*0xE2*/ "",
/*0xE3*/ "",
/*0xE4*/ "",
/*0xE5*/ "",
/*0xE6*/ "",
/*0xE7*/ "",
/*0xE8*/ "INX",
/*0xE9*/ "",
/*0xEA*/ "",
/*0xEB*/ "",
/*0xEC*/ "",
/*0xED*/ "",
/*0xEE*/ "",
/*0xEF*/ "",
/*0xF0*/ "",
/*0xF1*/ "",
/*0xF2*/ "",
/*0xF3*/ "",
/*0xF4*/ "",
/*0xF5*/ "",
/*0xF6*/ "",
/*0xF7*/ "",
/*0xF8*/ "",
/*0xF9*/ "",
/*0xFA*/ "",
/*0xFB*/ "",
/*0xFC*/ "",
/*0xFD*/ "",
/*0xFE*/ "",
/*0xFF*/ ""
};

char addressing[256][4] = {
/*0x00*/ "",
/*0x01*/ "",
/*0x02*/ "",
/*0x03*/ "",
/*0x04*/ "",
/*0x05*/ "",
/*0x06*/ "",
/*0x07*/ "",
/*0x08*/ "",
/*0x09*/ "",
/*0x0A*/ "IMP",
/*0x0B*/ "",
/*0x0C*/ "",
/*0x0D*/ "",
/*0x0E*/ "",
/*0x0F*/ "",
/*0x10*/ "",
/*0x11*/ "",
/*0x12*/ "",
/*0x13*/ "",
/*0x14*/ "",
/*0x15*/ "",
/*0x16*/ "",
/*0x17*/ "",
/*0x18*/ "",
/*0x19*/ "",
/*0x1A*/ "",
/*0x1B*/ "",
/*0x1C*/ "",
/*0x1D*/ "",
/*0x1E*/ "",
/*0x1F*/ "",
/*0x20*/ "I??",
/*0x21*/ "",
/*0x22*/ "",
/*0x23*/ "",
/*0x24*/ "",
/*0x25*/ "",
/*0x26*/ "",
/*0x27*/ "",
/*0x28*/ "",
/*0x29*/ "",
/*0x2A*/ "",
/*0x2B*/ "",
/*0x2C*/ "",
/*0x2D*/ "",
/*0x2E*/ "",
/*0x2F*/ "",
/*0x30*/ "",
/*0x31*/ "",
/*0x32*/ "",
/*0x33*/ "",
/*0x34*/ "",
/*0x35*/ "",
/*0x36*/ "",
/*0x37*/ "",
/*0x38*/ "",
/*0x39*/ "",
/*0x3A*/ "???",
/*0x3B*/ "",
/*0x3C*/ "",
/*0x3D*/ "",
/*0x3E*/ "",
/*0x3F*/ "",
/*0x40*/ "",
/*0x41*/ "",
/*0x42*/ "",
/*0x43*/ "",
/*0x44*/ "",
/*0x45*/ "",
/*0x46*/ "",
/*0x47*/ "",
/*0x48*/ "",
/*0x49*/ "",
/*0x4A*/ "",
/*0x4B*/ "",
/*0x4C*/ "ABS",
/*0x4D*/ "",
/*0x4E*/ "",
/*0x4F*/ "",
/*0x50*/ "",
/*0x51*/ "",
/*0x52*/ "",
/*0x53*/ "",
/*0x54*/ "",
/*0x55*/ "",
/*0x56*/ "",
/*0x57*/ "",
/*0x58*/ "",
/*0x59*/ "",
/*0x5A*/ "",
/*0x5B*/ "",
/*0x5C*/ "",
/*0x5D*/ "",
/*0x5E*/ "",
/*0x5F*/ "",
/*0x60*/ "I??",
/*0x61*/ "",
/*0x62*/ "",
/*0x63*/ "",
/*0x64*/ "",
/*0x65*/ "",
/*0x66*/ "",
/*0x67*/ "",
/*0x68*/ "",
/*0x69*/ "",
/*0x6A*/ "",
/*0x6B*/ "",
/*0x6C*/ "",
/*0x6D*/ "",
/*0x6E*/ "",
/*0x6F*/ "",
/*0x70*/ "",
/*0x71*/ "",
/*0x72*/ "",
/*0x73*/ "",
/*0x74*/ "",
/*0x75*/ "",
/*0x76*/ "",
/*0x77*/ "",
/*0x78*/ "",
/*0x79*/ "",
/*0x7A*/ "",
/*0x7B*/ "",
/*0x7C*/ "",
/*0x7D*/ "",
/*0x7E*/ "",
/*0x7F*/ "",
/*0x80*/ "",
/*0x81*/ "",
/*0x82*/ "",
/*0x83*/ "",
/*0x84*/ "APG",
/*0x85*/ "",
/*0x86*/ "",
/*0x87*/ "",
/*0x88*/ "IMP",
/*0x89*/ "",
/*0x8A*/ "",
/*0x8B*/ "",
/*0x8C*/ "",
/*0x8D*/ "",
/*0x8E*/ "",
/*0x8F*/ "",
/*0x90*/ "",
/*0x91*/ "",
/*0x92*/ "",
/*0x93*/ "",
/*0x94*/ "",
/*0x95*/ "",
/*0x96*/ "",
/*0x97*/ "",
/*0x98*/ "",
/*0x99*/ "",
/*0x9A*/ "",
/*0x9B*/ "",
/*0x9C*/ "",
/*0x9D*/ "",
/*0x9E*/ "",
/*0x9F*/ "",
/*0xA0*/ "",
/*0xA1*/ "",
/*0xA2*/ "",
/*0xA3*/ "",
/*0xA4*/ "",
/*0xA5*/ "",
/*0xA6*/ "",
/*0xA7*/ "",
/*0xA8*/ "",
/*0xA9*/ "IMM",
/*0xAA*/ "",
/*0xAB*/ "",
/*0xAC*/ "",
/*0xAD*/ "",
/*0xAE*/ "",
/*0xAF*/ "",
/*0xB0*/ "",
/*0xB1*/ "",
/*0xB2*/ "",
/*0xB3*/ "",
/*0xB4*/ "",
/*0xB5*/ "",
/*0xB6*/ "",
/*0xB7*/ "",
/*0xB8*/ "",
/*0xB9*/ "",
/*0xBA*/ "",
/*0xBB*/ "",
/*0xBC*/ "",
/*0xBD*/ "",
/*0xBE*/ "",
/*0xBF*/ "",
/*0xC0*/ "",
/*0xC1*/ "",
/*0xC2*/ "",
/*0xC3*/ "",
/*0xC4*/ "",
/*0xC5*/ "",
/*0xC6*/ "",
/*0xC7*/ "",
/*0xC8*/ "",
/*0xC9*/ "",
/*0xCA*/ "IMP",
/*0xCB*/ "",
/*0xCC*/ "",
/*0xCD*/ "",
/*0xCE*/ "",
/*0xCF*/ "",
/*0xD0*/ "",
/*0xD1*/ "",
/*0xD2*/ "",
/*0xD3*/ "",
/*0xD4*/ "",
/*0xD5*/ "",
/*0xD6*/ "",
/*0xD7*/ "",
/*0xD8*/ "IMP",
/*0xD9*/ "",
/*0xDA*/ "",
/*0xDB*/ "",
/*0xDC*/ "",
/*0xDD*/ "",
/*0xDE*/ "",
/*0xDF*/ "",
/*0xE0*/ "",
/*0xE1*/ "",
/*0xE2*/ "",
/*0xE3*/ "",
/*0xE4*/ "",
/*0xE5*/ "",
/*0xE6*/ "",
/*0xE7*/ "",
/*0xE8*/ "IMP",
/*0xE9*/ "",
/*0xEA*/ "",
/*0xEB*/ "",
/*0xEC*/ "",
/*0xED*/ "",
/*0xEE*/ "",
/*0xEF*/ "",
/*0xF0*/ "",
/*0xF1*/ "",
/*0xF2*/ "",
/*0xF3*/ "",
/*0xF4*/ "",
/*0xF5*/ "",
/*0xF6*/ "",
/*0xF7*/ "",
/*0xF8*/ "",
/*0xF9*/ "",
/*0xFA*/ "",
/*0xFB*/ "",
/*0xFC*/ "",
/*0xFD*/ "",
/*0xFE*/ "",
/*0xFF*/ ""
};
